from .bandit import simulateOAB, optimalOAB
from .dodge.dodge_sequential import sequentialDesign
from .dodge.dodge_single import SSPlanBinomial, SSPlanHyper, SSPlanPoisson
from .generic import findPlan, findPlanApprox
from .oc import OperatingCharacteristics2c
